unit MWconnU;

// Please, don't delete this comment. \\
(*
  Copyright Owner: coltishWARE (Kevin Niehage)
  Copyright Year : 2007

  Unit   : MWconnU (platform dependant)
  Version: 0.4c

  Contact E-Mail            : development@coltishWARE.com
  Contact Mobile            : +49-(0)-177-69-4321-8
  Contact Telephone Berlin  : +49-(0)-30-69-20-96-78
  Contact Telephone Cottbus : +49-(0)-355-28925-88-17-55
  Contact Telephone Mannheim: +49-(0)-621-483-4664
*)
// Please, don't delete this comment. \\

(*
  Description:

  This unit contains the shared memory interface
  implementation for the application MWconn by
  Markus B. Weber.

  It is a plain C to Pascal translation.
*)

(*
  Change Log:

  [Version 0.4c] (21.12.2007: versioning release)
  - moved compiler switching to seperate include file (MWconnU.inc)
  - structures are now references by their version (and not the version of MWconn)
  - moved structure versions to seperate include files (MWconnU[N].inc)

  [Version 0.3c] (11.12.2007: 3.10 release)
  - renamed types and compiler switches

  [Version 0.3b] (16.11.2007: 3.9C release)
  - support for 3.9C (not just 3.9 stable)

  [Version 0.2c] (13.11.2007: extended release)
  - calling method split into three methods (shared memory accessible now)
  - command constants introduced

  [Version 0.1c] (13.11.2007: initial release)
  - structure definition translated from C to Pascal
  - calling method written
*)

interface

uses
  Windows,
  SysUtils;

const
  cCopyrightOwner = 'coltishWARE (Kevin Niehage)';
  cCopyrightYear  = '2007';
  cName           = 'MWconnU';
  cReleaseDate    = '21.12.2007';
  cReleaseName    = 'versioning release';
  cVersion        = '0.4c';

// the different Structure-Versions are included here
{$I MWconnU2.inc}
{$I MWconnU1.inc}

// all compiler switches are included here
// additionally the type TMWconnIO is defined
// depending on which version to use
{$I MWconnU.inc}

type
  PMWconnIO = ^TMWconnIO;

const
  MWconnFile = 'MWCONN_IO';

const
  MWconn_Connect    = 'MWCONN_CONNECT';
  MWconn_Disconnect = 'MWCONN_DISCONNECT';
  MWconn_Reconnect  = 'MWCONN_RECONNECT';
  MWconn_End        = 'MWCONN_END';

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : Boolean;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;

implementation

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : Boolean;
begin
  Result := false;

  aFile := OpenFileMapping(FILE_MAP_ALL_ACCESS, false, MWconnFile);
  if (aFile <> 0) then
  begin
    try
      aMapping := MapViewOfFile(aFile, FILE_MAP_ALL_ACCESS, 0, 0, SizeOf(TMWconnIO));

      Result := (aMapping <> nil);
    except
      CloseHandle(aFile);
    end;
  end;
end;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);
begin
  if (aMapping <> nil) then
    UnmapViewOfFile(aMapping);
  if (aFile <> 0) then
    CloseHandle(aFile);

  aFile    := 0;
  aMapping := nil;
end;

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;
var
  lBuffer : PMWconnIO;
  lHandle : THandle;
begin
  Result := false;

  if OpenMWconnIO(lHandle, lBuffer) then
  begin
    try
      CopyMemory(@aData, lBuffer, SizeOf(TMWconnIO));

      Result := true;
    finally
      CloseMWconnIO(lHandle, lBuffer);
    end;
  end;
end;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;
const
  cSeperator = ':';
begin
  Result := cSeperator;

  if (aWaitDuration >= 0) then
    Result := Result + IntToStr(aWaitDuration);
  Result := Result + cSeperator;

  if (aReadDuration >= 0) then
    Result := Result + IntToStr(aReadDuration);
  Result := Result + cSeperator;

  Result := Result + aCommand;
end;

end.
